# Generated by Django 2.1.5 on 2019-02-15 09:15

import datetime
from django.db import migrations, models
from django.utils.timezone import utc
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0009_provider_agency_api_authentication'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='dn_battery_pct',
            field=models.FloatField(null=True),
        ),
        migrations.AlterField(
            model_name='area',
            name='creation_date',
            field=models.DateTimeField(default=datetime.datetime(2012, 1, 1, 0, 0, tzinfo=utc)),
        ),
        migrations.AlterField(
            model_name='area',
            name='deletion_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='area',
            name='label',
            field=mds.models.UnboundedCharField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='eventrecord',
            name='event_type',
            field=mds.models.UnboundedCharField(choices=[('service_start', 'Service start'), ('trip_end', 'Trip end'), ('rebalance_drop_off', 'Rebalance drop_off'), ('maintenance_drop_off', 'Maintenance drop off'), ('cancel_reservation', 'Cancel reservation'), ('reserve', 'Reserve'), ('trip_start', 'Trip start'), ('trip_enter', 'Trip enter'), ('trip_leave', 'Trip leave'), ('register', 'Register'), ('low_battery', 'Low battery'), ('maintenance', 'Maintenance'), ('service_end', 'Service end'), ('rebalance_pick_up', 'Rebalance pick up'), ('maintenance_pick_up', 'Maintenance pick up'), ('deregister', 'Deregister'), ('telemetry', 'Received telemetry'), ('battery_ok', 'Battery OK')]),
        ),
        migrations.AlterField(
            model_name='polygon',
            name='deletion_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='polygon',
            name='label',
            field=mds.models.UnboundedCharField(blank=True, null=True),
        ),
    ]
