# Generated by Django 2.1.3 on 2019-01-22 09:25

import django.contrib.gis.db.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0003_record_timestamp'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventrecord',
            name='point',
            field=django.contrib.gis.db.models.fields.PointField(null=True, srid=4326),
        ),
    ]
