# Generated by Django 2.1.3 on 2019-01-25 10:14

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0005_auto_20190122_1300'),
    ]

    operations = [
        migrations.AddField(
            model_name='provider',
            name='device_category',
            field=mds.models.UnboundedCharField(choices=[('bike', 'Bike'), ('scooter', 'Scooter'), ('car', 'Car')], default='scooter'),
            preserve_default=False,
        ),
    ]
